@@ -1,8 +1,9 @@
-import React, { PureComponent } from 'react';
+import { PureComponent } from 'react';
+import * as React from 'react';
 
 import { isEmptyObject, SelectableValue, VariableRefresh } from '@grafana/data';
 import { getBackendSrv } from '@grafana/runtime';
-import { Button, ClipboardButton, Field, Input, LinkButton, Modal, Select, Spinner } from '@grafana/ui';
+import { Button, ClipboardButton, Field, Input, LinkButton, Modal, Select, Spinner, Stack } from '@grafana/ui';
 import { t, Trans } from 'app/core/internationalization';
 import { getTimeSrv } from 'app/features/dashboard/services/TimeSrv';
 import { DashboardModel, PanelModel } from 'app/features/dashboard/state';
@@ -239,14 +240,14 @@ export class ShareSnapshot extends PureComponent<Props, State> {
     return (
       <>
         <div>
-          <p className="share-modal-info-text">
+          <p>
             <Trans i18nKey="share-modal.snapshot.info-text-1">
               A snapshot is an instant way to share an interactive dashboard publicly. When created, we strip sensitive
               data like queries (metric, template, and annotation) and panel links, leaving only the visible metric data
               and series names embedded in your dashboard.
             </Trans>
           </p>
-          <p className="share-modal-info-text">
+          <p>
             <Trans i18nKey="share-modal.snapshot.info-text-2">
               Keep in mind, your snapshot <em>can be viewed by anyone</em> that has the link and can access the URL.
               Share wisely.
@@ -290,7 +291,7 @@ export class ShareSnapshot extends PureComponent<Props, State> {
     const { snapshotUrl } = this.state;
 
     return (
-      <>
+      <Stack direction="column" gap={0}>
         <Field label={t('share-modal.snapshot.url-label', 'Snapshot URL')}>
           <Input
             id="snapshot-url-input"
@@ -304,26 +305,24 @@ export class ShareSnapshot extends PureComponent<Props, State> {
           />
         </Field>
 
-        <div className="pull-right" style={{ padding: '5px' }}>
+        <div style={{ alignSelf: 'flex-end', padding: '5px' }}>
           <Trans i18nKey="share-modal.snapshot.mistake-message">Did you make a mistake? </Trans>&nbsp;
           <LinkButton fill="text" target="_blank" onClick={this.deleteSnapshot}>
             <Trans i18nKey="share-modal.snapshot.delete-button">Delete snapshot.</Trans>
           </LinkButton>
         </div>
-      </>
+      </Stack>
     );
   }
 
   renderStep3() {
     return (
-      <div className="share-modal-header">
-        <p className="share-modal-info-text">
-          <Trans i18nKey="share-modal.snapshot.deleted-message">
-            The snapshot has been deleted. If you have already accessed it once, then it might take up to an hour before
-            before it is removed from browser caches or CDN caches.
-          </Trans>
-        </p>
-      </div>
+      <p>
+        <Trans i18nKey="share-modal.snapshot.deleted-message">
+          The snapshot has been deleted. If you have already accessed it once, then it might take up to an hour before
+          before it is removed from browser caches or CDN caches.
+        </Trans>
+      </p>
     );
   }
 
