@@ -1,17 +1,19 @@
-import React, { useEffect } from 'react';
+import { useEffect } from 'react';
+
 import { ConfirmModal } from '@grafana/ui';
+
 import { useDashboardRestore } from './useDashboardRestore';
 export interface RevertDashboardModalProps {
   hideModal: () => void;
   version: number;
 }
 
-export const RevertDashboardModal: React.FC<RevertDashboardModalProps> = ({ hideModal, version }) => {
+export const RevertDashboardModal = ({ hideModal, version }: RevertDashboardModalProps) => {
   // TODO: how should state.error be handled?
   const { state, onRestoreDashboard } = useDashboardRestore(version);
 
   useEffect(() => {
-    if (state.loading === false && state.value) {
+    if (!state.loading && state.value) {
       hideModal();
     }
   }, [state, hideModal]);
