@@ -1,6 +1,6 @@
 import { css } from '@emotion/css';
-import React, { createRef } from 'react';
-import SVG from 'react-inlinesvg';
+import { createRef } from 'react';
+import * as React from 'react';
 
 import { GrafanaTheme2 } from '@grafana/data';
 import {
@@ -15,6 +15,7 @@ import {
   useTheme2,
 } from '@grafana/ui';
 import { closePopover } from '@grafana/ui/src/utils/closePopover';
+import { SanitizedSVG } from 'app/core/components/SVG/SanitizedSVG';
 
 import { getPublicOrAbsoluteUrl } from '../resource';
 import { MediaType, ResourceFolderName, ResourcePickerSize } from '../types';
@@ -32,17 +33,24 @@ interface Props {
   name?: string;
   placeholder?: string;
   color?: string;
+  maxFiles?: number;
 }
 
 export const ResourcePicker = (props: Props) => {
-  const { value, src, name, placeholder, onChange, onClear, mediaType, folderName, size, color } = props;
+  const { value, src, name, placeholder, onChange, onClear, mediaType, folderName, size, color, maxFiles } = props;
 
   const styles = useStyles2(getStyles);
   const theme = useTheme2();
 
-  const pickerTriggerRef = createRef<any>();
+  const pickerTriggerRef = createRef<HTMLDivElement>();
   const popoverElement = (
-    <ResourcePickerPopover onChange={onChange} value={value} mediaType={mediaType} folderName={folderName} />
+    <ResourcePickerPopover
+      onChange={onChange}
+      value={value}
+      mediaType={mediaType}
+      folderName={folderName}
+      maxFiles={maxFiles}
+    />
   );
 
   let sanitizedSrc = src;
@@ -56,7 +64,7 @@ export const ResourcePicker = (props: Props) => {
 
   const renderSmallResourcePicker = () => {
     if (value && sanitizedSrc) {
-      return <SVG src={sanitizedSrc} className={styles.icon} style={{ ...colorStyle }} />;
+      return <SanitizedSVG src={sanitizedSrc} className={styles.icon} style={{ ...colorStyle }} />;
     } else {
       return (
         <LinkButton variant="primary" fill="text" size="sm">
@@ -70,10 +78,10 @@ export const ResourcePicker = (props: Props) => {
     <InlineFieldRow>
       <InlineField label={null} grow>
         <Input
-          value={name}
+          value={getDisplayName(src, name)}
           placeholder={placeholder}
           readOnly={true}
-          prefix={sanitizedSrc && <SVG src={sanitizedSrc} className={styles.icon} style={{ ...colorStyle }} />}
+          prefix={sanitizedSrc && <SanitizedSVG src={sanitizedSrc} className={styles.icon} style={{ ...colorStyle }} />}
           suffix={<Button icon="times" variant="secondary" fill="text" size="sm" onClick={onClear} />}
         />
       </InlineField>
@@ -96,7 +104,18 @@ export const ResourcePicker = (props: Props) => {
               />
             )}
 
-            <div ref={pickerTriggerRef} onClick={showPopper} className={styles.pointer}>
+            <div
+              ref={pickerTriggerRef}
+              className={styles.pointer}
+              onClick={showPopper}
+              onKeyDown={(e: React.KeyboardEvent) => {
+                if (e.key === 'Enter') {
+                  showPopper();
+                }
+              }}
+              role="button"
+              tabIndex={0}
+            >
               {size === ResourcePickerSize.SMALL && renderSmallResourcePicker()}
               {size === ResourcePickerSize.NORMAL && renderNormalResourcePicker()}
             </div>
@@ -107,6 +126,17 @@ export const ResourcePicker = (props: Props) => {
   );
 };
 
+// strip the SVG off icons in the icons folder
+function getDisplayName(src?: string, name?: string): string | undefined {
+  if (src?.startsWith('public/img/icons')) {
+    const idx = name?.lastIndexOf('.svg') ?? 0;
+    if (idx > 0) {
+      return name!.substring(0, idx);
+    }
+  }
+  return name;
+}
+
 const getStyles = (theme: GrafanaTheme2) => ({
   pointer: css`
     cursor: pointer;
