@@ -1,5 +1,4 @@
 import { css, cx } from '@emotion/css';
-import React from 'react';
 import Skeleton from 'react-loading-skeleton';
 
 import { GrafanaTheme2 } from '@grafana/data';
@@ -7,7 +6,7 @@ import { locationService, reportInteraction } from '@grafana/runtime';
 import { Badge, Icon, Stack, useStyles2 } from '@grafana/ui';
 import { SkeletonComponent, attachSkeleton } from '@grafana/ui/src/unstable';
 
-import { CatalogPlugin, PluginIconName, PluginListDisplayMode } from '../types';
+import { CatalogPlugin, PluginIconName } from '../types';
 
 import { PluginListItemBadges } from './PluginListItemBadges';
 import { PluginLogo } from './PluginLogo';
@@ -17,12 +16,10 @@ export const LOGO_SIZE = '48px';
 type Props = {
   plugin: CatalogPlugin;
   pathName: string;
-  displayMode?: PluginListDisplayMode;
 };
 
-function PluginListItemComponent({ plugin, pathName, displayMode = PluginListDisplayMode.Grid }: Props) {
+function PluginListItemComponent({ plugin, pathName }: Props) {
   const styles = useStyles2(getStyles);
-  const isList = displayMode === PluginListDisplayMode.List;
 
   const reportUserClickInteraction = () => {
     if (locationService.getSearchObject()?.q) {
@@ -30,11 +27,7 @@ function PluginListItemComponent({ plugin, pathName, displayMode = PluginListDis
     }
   };
   return (
-    <a
-      href={`${pathName}/${plugin.id}`}
-      className={cx(styles.container, { [styles.list]: isList })}
-      onClick={reportUserClickInteraction}
-    >
+    <a href={`${pathName}/${plugin.id}`} className={cx(styles.container)} onClick={reportUserClickInteraction}>
       <PluginLogo src={plugin.info.logos.small} className={styles.pluginLogo} height={LOGO_SIZE} alt="" />
       <h2 className={cx(styles.name, 'plugin-name')}>{plugin.name}</h2>
       <div className={cx(styles.content, 'plugin-content')}>
@@ -48,15 +41,11 @@ function PluginListItemComponent({ plugin, pathName, displayMode = PluginListDis
   );
 }
 
-const PluginListItemSkeleton: SkeletonComponent<Pick<Props, 'displayMode'>> = ({
-  displayMode = PluginListDisplayMode.Grid,
-  rootProps,
-}) => {
+const PluginListItemSkeleton: SkeletonComponent = ({ rootProps }) => {
   const styles = useStyles2(getStyles);
-  const isList = displayMode === PluginListDisplayMode.List;
 
   return (
-    <div className={cx(styles.container, { [styles.list]: isList })} {...rootProps}>
+    <div className={cx(styles.container)} {...rootProps}>
       <Skeleton
         containerClassName={cx(
           styles.pluginLogo,
@@ -110,27 +99,6 @@ export const getStyles = (theme: GrafanaTheme2) => {
         background: theme.colors.emphasize(theme.colors.background.secondary, 0.03),
       },
     }),
-    list: css({
-      rowGap: 0,
-
-      '> img': {
-        alignSelf: 'start',
-      },
-
-      '> .plugin-content': {
-        minHeight: 0,
-        gridArea: '2 / 2 / 4 / 3',
-
-        '> p': {
-          margin: theme.spacing(0, 0, 0.5, 0),
-        },
-      },
-
-      '> .plugin-name': {
-        alignSelf: 'center',
-        gridArea: '1 / 2 / 2 / 3',
-      },
-    }),
     pluginType: css({
       gridArea: '1 / 3 / 2 / 4',
       color: theme.colors.text.secondary,
